import 'package:flutter/material.dart';
import 'package:xinlake_platform/xinlake_platform.dart';

class PickFilesDemo extends StatefulWidget {
  const PickFilesDemo({Key? key}) : super(key: key);

  @override
  State<PickFilesDemo> createState() => _PickFilesState();
}

class _PickFilesState extends State<PickFilesDemo> {
  bool _multiSelection = false;
  String _mimeType = "*/*";
  AndroidCacheDir _cacheDir = AndroidCacheDir.none;
  bool _focusCache = false;
  String _openPath = "";
  String _defaultPath = "";
  String _filterName = "All files";
  String _filterPattern = "*.*";

  // result
  List<String>? _pathList;

  void _pickFiles() async {
    var pathList = await XinlakePlatform.pickFiles(
      multiSelection: _multiSelection,
      mimeType: _mimeType,
      cacheDir: _cacheDir,
      forceCache: _focusCache,
      openPath: _openPath,
      defaultPath: _defaultPath,
      filterName: _filterName,
      filterPattern: _filterPattern,
    );

    setState(() => _pathList = pathList);
  }

  Widget _buildResults() {
    final style = Theme.of(context).textTheme.caption;
    // empty view
    if (_pathList == null) {
      return Padding(
        padding: const EdgeInsets.all(10),
        child: Text(
          "NULL",
          textScaleFactor: 2,
          style: style,
        ),
      );
    } else if (_pathList!.isEmpty) {
      return Padding(
        padding: const EdgeInsets.all(10),
        child: Text(
          "EMPTY",
          textScaleFactor: 2,
          style: style,
        ),
      );
    }

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        Padding(
          padding: const EdgeInsets.symmetric(vertical: 10),
          child: Text(
            "${_pathList!.length} ${_pathList!.length > 1 ? "files" : "file"} selected",
            style: style,
          ),
        ),
        ..._pathList!.map<Widget>((path) {
          return Padding(
            padding: const EdgeInsets.symmetric(vertical: 5),
            child: Text(
              path,
              maxLines: 3,
              overflow: TextOverflow.ellipsis,
            ),
          );
        }).toList(),
      ],
    );
  }

  Widget _buildParameter() {
    return Column(
      children: [
        // all platform
        Row(
          children: [
            const Expanded(
              child: Text("Multi selection"),
            ),
            Expanded(
              child: Container(
                alignment: Alignment.centerLeft,
                padding: const EdgeInsets.symmetric(vertical: 5),
                child: Switch(
                  value: _multiSelection,
                  onChanged: (value) => setState(() => _multiSelection = value),
                ),
              ),
            ),
          ],
        ),
        // android
        Row(
          children: const [
            Expanded(child: Divider(height: 1)),
            Text(
              "ANDROID",
              style: TextStyle(fontStyle: FontStyle.italic),
            ),
          ],
        ),
        Row(
          children: [
            Expanded(
              child: TextFormField(
                decoration: const InputDecoration(labelText: "Mime type"),
                initialValue: _mimeType,
                onChanged: (value) => _mimeType = value,
              ),
            ),
          ],
        ),
        Column(
          mainAxisSize: MainAxisSize.min,
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: [
            const SizedBox(height: 20),
            Text(
              "Cache dir",
              style: Theme.of(context).textTheme.caption,
            ),
            const SizedBox(height: 5),
            DropdownButton<AndroidCacheDir>(
              value: _cacheDir,
              isDense: true,
              underline: Container(
                height: 2,
                color: Theme.of(context).colorScheme.background,
              ),
              onChanged: (newValue) {
                setState(() => _cacheDir = newValue!);
              },
              items: AndroidCacheDir.values.map<DropdownMenuItem<AndroidCacheDir>>((item) {
                return DropdownMenuItem<AndroidCacheDir>(
                  value: item,
                  child: Text(item.toString()),
                );
              }).toList(),
            ),
            const SizedBox(height: 20),
          ],
        ),
        Row(
          children: [
            const Expanded(
              child: Text("Focus cache"),
            ),
            Expanded(
              child: Container(
                alignment: Alignment.centerLeft,
                padding: const EdgeInsets.symmetric(vertical: 5),
                child: Switch(
                  value: _focusCache,
                  onChanged: (value) => setState(() => _focusCache = value),
                ),
              ),
            ),
          ],
        ),
        // windows
        Row(
          children: const [
            Expanded(child: Divider(height: 1)),
            Text(
              "WINDOWS",
              style: TextStyle(fontStyle: FontStyle.italic),
            ),
          ],
        ),
        Row(
          children: [
            Expanded(
              child: TextFormField(
                decoration: const InputDecoration(labelText: "Open path"),
                initialValue: _openPath,
                onChanged: (value) => _openPath = value,
              ),
            ),
            const SizedBox(width: 10),
            Expanded(
              child: TextFormField(
                decoration: const InputDecoration(labelText: "Default path"),
                initialValue: _defaultPath,
                onChanged: (value) => _defaultPath = value,
              ),
            ),
          ],
        ),
        Row(
          children: [
            Expanded(
              child: TextFormField(
                decoration: const InputDecoration(labelText: "Filter name"),
                initialValue: _filterName,
                onChanged: (value) => _filterName = value,
              ),
            ),
            const SizedBox(width: 10),
            Expanded(
              child: TextFormField(
                decoration: const InputDecoration(labelText: "Filter pattern"),
                initialValue: _filterPattern,
                onChanged: (value) => _filterPattern = value,
              ),
            ),
          ],
        ),
      ],
    );
  }

  @override
  Widget build(BuildContext context) {
    return SingleChildScrollView(
      padding: const EdgeInsets.all(10),
      child: Column(
        children: [
          _buildParameter(),
          const SizedBox(height: 10),
          Row(
            children: [
              Expanded(
                child: ElevatedButton(
                  child: const Text("pickFiles"),
                  onPressed: _pickFiles,
                ),
              ),
            ],
          ),
          _buildResults(),
        ],
      ),
    );
  }
}
