#include <windows.h>
#include <shobjidl.h> 

#include <string>
#include <vector>
#include <list>

std::string Utf8FromUtf16(const wchar_t* utf16_string);
std::wstring WStringFromString(const std::string& string);

/// <summary>
/// 
/// </summary>
/// <param name="filterName">
/// "filter description"
/// </param>
/// <param name="filterPattern">
/// "*.c; *.cpp"
/// </param>
/// <returns></returns>
std::list<std::string> openFileDialog(bool multiSelection, std::string openPath, std::string defaultPath,
    std::string filterName, std::string filterPattern) {

    std::list<std::string> pathList;
    HRESULT hr = CoInitializeEx(NULL, COINIT_APARTMENTTHREADED | COINIT_DISABLE_OLE1DDE);
    if (SUCCEEDED(hr)) {

        // Create the FileOpenDialog object.
        IFileOpenDialog* pFileOpen;
        hr = CoCreateInstance(CLSID_FileOpenDialog, NULL, CLSCTX_ALL,
            IID_IFileOpenDialog, reinterpret_cast<void**>(&pFileOpen));
        if (SUCCEEDED(hr)) {

            // set options
            DWORD options;
            hr = pFileOpen->GetOptions(&options);
            if (SUCCEEDED(hr)) {
                options = multiSelection
                    ? (options | FOS_ALLOWMULTISELECT | FOS_FORCEFILESYSTEM)
                    : (options | FOS_FORCEFILESYSTEM);
                hr = pFileOpen->SetOptions(options);
                if (SUCCEEDED(hr)) {

                    // set open folder or default folder, continue 
                    if (!openPath.empty()) {
                        IShellItem* pFolder;
                        std::wstring wOpenPath = WStringFromString(openPath);
                        hr = SHCreateItemFromParsingName(wOpenPath.c_str(), NULL, IID_PPV_ARGS(&pFolder));
                        if (SUCCEEDED(hr)) {
                            hr = pFileOpen->SetFolder(pFolder);
                        }
                    } else if (!defaultPath.empty()) {
                        IShellItem* pFolder;
                        std::wstring wDefaultPath = WStringFromString(defaultPath);
                        hr = SHCreateItemFromParsingName(wDefaultPath.c_str(), NULL, IID_PPV_ARGS(&pFolder));
                        if (SUCCEEDED(hr)) {
                            hr = pFileOpen->SetDefaultFolder(pFolder);
                        }
                    }

                    // set types
                    std::wstring wFilterName = WStringFromString(filterName);
                    std::wstring wFilterPattern = WStringFromString(filterPattern);
                    COMDLG_FILTERSPEC filters[] = {
                        { wFilterName.c_str(), wFilterPattern.c_str() },
                    };
                    hr = pFileOpen->SetFileTypes(ARRAYSIZE(filters), filters);
                    if (SUCCEEDED(hr)) {

                        // Show the Open dialog box.
                        HWND hwnd = GetActiveWindow();
                        hr = pFileOpen->Show(hwnd);
                        if (SUCCEEDED(hr)) {

                            // Get the results
                            //if (multiSelection) {
                            IShellItemArray* pResults;
                            hr = pFileOpen->GetResults(&pResults);
                            if (SUCCEEDED(hr)) {

                                DWORD pathCount;
                                hr = pResults->GetCount(&pathCount);
                                if (SUCCEEDED(hr)) {

                                    // set result list
                                    for (DWORD i = 0; i < pathCount; ++i) {
                                        IShellItem* pResult;
                                        hr = pResults->GetItemAt(i, &pResult);
                                        if (SUCCEEDED(hr)) {

                                            // set result
                                            PWSTR filePath;
                                            hr = pResult->GetDisplayName(SIGDN_FILESYSPATH, &filePath);
                                            if (SUCCEEDED(hr)) {

                                                std::string path = Utf8FromUtf16(filePath);
                                                pathList.push_back(path);
                                                CoTaskMemFree(filePath);
                                            }

                                            pResult->Release();
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            pFileOpen->Release();
        }

        CoUninitialize();
    }

    return pathList;
}

std::string Utf8FromUtf16(const wchar_t* utf16_string) {
    if (utf16_string == nullptr) {
        return std::string();
    }

    int target_length = ::WideCharToMultiByte(CP_UTF8, WC_ERR_INVALID_CHARS, utf16_string,
        -1, nullptr, 0, nullptr, nullptr);
    if (target_length == 0) {
        return std::string();
    }

    std::string utf8_string;
    utf8_string.resize(target_length);
    int converted_length = ::WideCharToMultiByte(
        CP_UTF8, WC_ERR_INVALID_CHARS, utf16_string,
        -1, utf8_string.data(), target_length, nullptr, nullptr);

    if (converted_length == 0) {
        return std::string();
    }

    return utf8_string;
}

std::wstring WStringFromString(const std::string& string) {
    std::vector<wchar_t> buff(
        MultiByteToWideChar(CP_ACP, 0, string.c_str(), (int)(string.size() + 1), 0, 0)
    );

    MultiByteToWideChar(CP_ACP, 0, string.c_str(), (int)(string.size() + 1), &buff[0], (int)(buff.size()));

    return std::wstring(&buff[0]);
}
